@@ -1,12 +1,14 @@
 #!/usr/bin/env python
 
+# pylint: disable=superfluous-parens
+
 '''
 Provides a treap module - specifically, the class that describes the overall treap, not the nodes.
 
 A treap is a datastructure that's kind of half way between a heap and a binary search tree
 '''
 
-## Editor width - you must be at least this tall to enjoy this ride ################################################################
+# Editor width - you must be at least this tall to enjoy this ride #################################################################
 
 # Appears to be an excellent reference on treaps in java:
 # http://users.cis.fiu.edu/~weiss/dsaajava/Code/DataStructures
@@ -23,15 +25,11 @@ A treap is a datastructure that's kind of half way between a heap and a binary s
 
 
 import sys
-#mport copy
 import math
-#mport time
-#mport functools
 
 # actually, it's a little faster with the standard random module than the lcgrng module; although random is a more time
 # consuming algorithm, it's coded in C.
 import random as random_module
-#import lcgrng as random_module
 
 PRIORITY_SIZE = 0x7fffffff
 
@@ -41,6 +39,11 @@ PRIORITY_SIZE = 0x7fffffff
 SENTINEL = object()
 
 
+def make_used(var):
+    """Persuade linters that 'var' is used."""
+    assert True or var
+
+
 class treap_node(object):
     '''Hold a single node of a treap'''
 
@@ -112,8 +115,10 @@ class treap_node(object):
             elif key > current.key:
                 current = current.right
             else:
-                # equal
-                return current
+                # equal: break out of the loop and return
+                break
+
+        return current
 
     def check_invariants(self):
         '''Check the tree and heap invariants'''
@@ -244,7 +249,6 @@ class treap_node(object):
 
     def __str__(self):
         return '%s/%s/%s' % (self.key, self.priority, self.value)
-        #return '%s/%s' % (self.key, self.value)
 
     def find_min_node(self):
         '''Find the lowest node in the treap'''
@@ -270,6 +274,7 @@ class treap_node(object):
 
         return current
 
+
 # Each element is stored in a node in a tree, and each node contains an integer and two references, one to the left
 # subtree and one to the right subtree.
 
@@ -312,16 +317,22 @@ class treap(object):
 
     # __bool__ is the python 3 name of the special method, while __nonzero__ is the python 2 name
     def __bool__(self):
-        if self.root is None:
-            return False
-        else:
-            return True
+        return self.root is not None
 
     __nonzero__ = __bool__
 
     def __len__(self):
         return self.length
 
+    def _slow_len(self):
+        '''Compute the length of the treap in a slow but accurate way'''
+        count = 0
+
+        for _ in self.values():
+            count += 1
+
+        return count
+
     def __setitem__(self, key, value, priority=None):
         '''Insert a node in the treap'''
         if self.root is None:
@@ -384,11 +395,38 @@ class treap(object):
             elif key > current.key:
                 current = current.right
             else:
-                # equal
-                return current.value
+                # equal: break out of the loop and return
+                break
+
+        return current.value
+
+    def get_key(self, key):
+        '''Look up a _key_ in the treap by key: return the key'''
+        current = self.root
+
+        while True:
+            if current is None:
+                raise KeyError
+            elif key < current.key:
+                current = current.left
+            elif key > current.key:
+                current = current.right
+            else:
+                # equal, break out of the loop and return
+                break
+
+        return current.key
 
     find = __getitem__
 
+    def __contains__(self, key):
+        try:
+            make_used(self[key])
+        except KeyError:
+            return False
+        else:
+            return True
+
     def find_min(self):
         '''Find the lowest key in the treap'''
         current = self.root
@@ -485,15 +523,13 @@ class treap(object):
         '''Check the tree invariant'''
         if self.root is None:
             return True
-        else:
-            return self.root.check_tree_invariant()
+        return self.root.check_tree_invariant()
 
     def check_heap_invariant(self):
         '''Check the heap invariant'''
         if self.root is None:
             return True
-        else:
-            return self.root.check_heap_invariant()
+        return self.root.check_heap_invariant()
 
     def depth(self):
         '''Return the depth of the treap (tree)'''
@@ -506,10 +542,7 @@ class treap(object):
                 # pylint: disable=R0913
                 # R0913: We need a bunch of arguments
                 '''Check our maximum so far against the current node - updating as needed'''
-                dummy = node
-                dummy = key
-                dummy = value
-                dummy = from_left
+                make_used([node, key, value, from_left])
                 if depth > self.max:
                     self.max = depth
 
@@ -533,9 +566,7 @@ class treap(object):
                 '''Check our maximums so far against the current node - updating as needed'''
                 # pylint: disable=R0913
                 # R0913: We need a bunch of arguments
-                dummy = key
-                dummy = value
-                dummy = from_left
+                make_used([key, value, from_left])
                 if depth > self.depth_max:
                     self.depth_max = depth
                 str_node = str(node)
@@ -564,31 +595,29 @@ class treap(object):
                 '''Add a node to the tree'''
                 # pylint: disable=R0913
                 # R0913: We need a bunch of arguments
-                dummy = key
-                dummy = value
+                make_used([key, value])
                 self.tree[depth][from_left] = str(node)
 
         if self.root is None:
             # empty output for an empty tree
             return ''
-        else:
-            pretree = []
-            depth, field_use_width = self._depth_and_field_width()
-            field_use_width += 1
-            for level in range(depth):
-                string = '_' * (field_use_width - 1)
-                pretree.append([string] * 2 ** level)
-            desc = Desc(pretree)
-            self.root.detailed_inorder_traversal(desc.update)
-            result = []
-            widest = 2 ** (depth - 1) * field_use_width
-            for level in range(depth):
-                two_level = 2 ** level
-                field_avail_width = widest / two_level
-                string = ''.join([center(x, field_use_width, field_avail_width) for x in desc.tree[level]])
-                # this really isn't useful for more than dozen values or so, and that without priorities printed
-                result.append(('%2d ' % level) + string)
-            return '\n'.join(result)
+        pretree = []
+        depth, field_use_width = self._depth_and_field_width()
+        field_use_width += 1
+        for level in range(depth):
+            string = '_' * (field_use_width - 1)
+            pretree.append([string] * 2 ** level)
+        desc = Desc(pretree)
+        self.root.detailed_inorder_traversal(desc.update)
+        result = []
+        widest = 2 ** (depth - 1) * field_use_width
+        for level in range(depth):
+            two_level = 2 ** level
+            field_avail_width = widest / two_level
+            string = ''.join([center(x, field_use_width, field_avail_width) for x in desc.tree[level]])
+            # this really isn't useful for more than dozen values or so, and that without priorities printed
+            result.append(('%2d ' % level) + string)
+        return '\n'.join(result)
 
     class state_class(object):
         # pylint: disable=R0903
@@ -678,5 +707,3 @@ class treap(object):
                     stack.append(self.state_class('V', state.node))
                     if state.node.right is not None:
                         stack.append(self.state_class('R', state.node.right))
-
-
